# -*- encoding: utf-8 -*-

import os
import re


class Disk(object):
    """
    Class for disk operations.
    """
    @staticmethod
    def free_disk_space(path):
        """
        Calculate free disk space for given path.

        Returned available_bytes or None when path not exists or writable.
        """
        if Disk.path_writeble(path):
            stats = os.statvfs(path)

            available_bytes = (stats.f_bavail * stats.f_frsize)
            return available_bytes
        else:
            return None

    @staticmethod
    def path_writeble(path):
        """
        Check if a given path is writable for user.
        """
        if os.path.exists(path):
            return os.access(path, os.W_OK)
        elif os.access(os.path.dirname(path), os.W_OK) and \
                os.access(path, os.W_OK):
            return True
        else:
            return False

    @staticmethod
    def sizeof_fmt(num, suffix="B"):
        """
        Convert number of bytes into human readable format.
        Source: https://web.archive.org/web/20111010015624/ \
                http://blogmag.net/blog/read/38/Print_human_readable_file_size
        """
        for unit in ["", "Ki", "Mi", "Gi", "Ti", "Pi", "Ei", "Zi"]:
            if abs(num) < 1024.0:
                return "%3.1f %s%s" % (num, unit, suffix)
            num /= 1024.0

        return "%.1f %s%s" % (num, "Yi", suffix)

    @staticmethod
    def create_capture_folder(path, samp_rate, freq, tag):
        """
        Create capture folder with spezific name scheme.
        """
        if not Disk.path_writeble(path):
            return None
        else:
            su_folders = Disk.get_su_created_folders(path)
            last_id = Disk.last_used_id(su_folders)
            new_id = Disk.create_new_su_id(last_id + 1)

            folder_string = new_id + "-" + \
                            str(int(freq)) + "MHz-" + \
                            str(int(samp_rate)) + "MHz-" + \
                            tag.replace(" ", "_")

            new_folder_path = path + "/" + folder_string
            new_folder_path = new_folder_path.replace("//", "/")
            os.mkdir(new_folder_path)

            return new_folder_path

    @staticmethod
    def get_su_created_folders(path):
        """
        Return all su folders in given path.
        """
        folders = []
        for folder in os.listdir(path):
            folder_path = path + "/" + folder
            folder_path = folder_path.replace("//", "/")
            if os.path.isdir(folder_path):
                folders.append(folder_path)

        su_folders = []
        for full_folder_path in folders:
            last_folder = os.path.basename(os.path.normpath(full_folder_path))
            if Disk.is_su_folder(last_folder):
                su_folders.append(full_folder_path)

        return su_folders

    @staticmethod
    def is_su_folder(folder_name):
        """
        Check given folder for matching su folder structure.
        """
        if re.search(r"^\d{5}-\d+.*", folder_name):
            return True
        else:
            return False

    @staticmethod
    def last_used_id(folders):
        """
        Return last used su id in given folder.
        """
        highest_id = 0
        for folder in folders:
            last_folder = os.path.basename(os.path.normpath(folder))
            match = re.search(r"^\d{5}", last_folder)
            if match:
                new_id = int(match.group())
                if new_id > highest_id:
                    highest_id = new_id

        return highest_id

    @staticmethod
    def create_folder(path):
        """
        Create
        """
        if not os.path.exists(path):
            os.makedirs(path)

    @staticmethod
    def create_new_su_id(new_id):
        """
        Create new su path id.
        """
        max_length = 5
        id_length = len(str(new_id))

        return ("0"*(max_length-id_length)) + str(new_id)
